# encoding=utf8
"""
制作*.whl的安装文件
"""
__version__ = "2.0.21"

import traceback

extras_require = {  # 额外依赖项，安装方法“pip install yangke[Database]”
    "Database": [
        'pymysql>=0.9.3',
        'DBUtils>=1.3',  # 用于创建数据库连接池
        'cryptography>=3.1.1',  # 用于mysql8以上版本的连接加密，必须安装，否则无法解析密码
        'python-docx',
        'mysql-connector-python>=8.0.24',
        'sqlalchemy<=1.4.39',  # 2.0版本的sqlalchemy存在很多兼容性问题，还需要进一步解决
    ],
    "windows": [
        'pypiwin32',
    ],
    "web": [
        'flask>=1.1.2',
        'Flask-Cors>=3.0.8',
        'waitress',
    ],
    "Stock": [
        # 'torch>=1.4.0', # pytorch需要单独安装，pypi里版本太老
        'tushare>=1.2.48',
        'scrapy>=2.0.0',
        # 'sxtwl>=1.0.7',  # 该库是用于农历节假日计算的，从2022年开始已经停止更新了，因此新版python不适用
        'pymysql>=0.9.3',
        'DBUtils>=1.3',
        'selenium>=3.141.0',
        'mysql-connector-python>=8.0.24',
        'selenium>=3.141.0',
        'sqlalchemy>=1.3.23',
    ],
    "ImageRecognition": [
        'opencv-python>=4.2.0',
        'cmake>=3.18.2',
        'boost',
        # 'dlib>=19.17.0',  # dlib需要单独安装，涉及到cmake和boost
        'pillow>=7.0.0',
        'requests>=2.22.0',
        # 'tensorflow>=2.8.0',
        # 'torch>=1.11.0',
        'optuna',
        'plotly',
        'paddlepaddle',

    ],

    "GameServer": [
        'twisted>=20.3.0',
        'flask>=1.1.2',
        'flask_cors>=3.0.8',
        'requests>=2.22.0',
        'gevent>=20.5.0',
        'gevent-websocket',
        'waitress>=1.4.4',
        'lxml',
    ],
    "Performance": [
        'iapws>=1.5.2',
        'pygame',
        # 'PyQt5',
        # 安装PyQt5时如果出现ERROR: Could not install packages due to an OSError: [Errno 13] Permission denied:
        # ‘C:\Users\zhangbin\AppData\Roaming\Python\Python39\site-packages\PyQt5\Qt5\bin\d3dcompiler_47.dll’，
        # 说明d3dcompiler_47.dll文件占用，禁止无关应用开机启动，重启安装尝试。
        # 'PyQtWebEngine',  # 用于显示pyecharts图形
        'PyQt6',
        "PyQt6-WebEngine",
        'PyQt6-QScintilla'
        'PyQt6-WebEngine',
        # 'PySide6'
    ],

}

module_name_list = list(extras_require.keys())


def info():
    from yangke.common.config import printInColor
    printInColor(" module 'yangke' installed successfully ",
                 color_fg='white', color_bg='cyan', mode=1)
    printInColor(" version is {} ".format(__version__),
                 color_fg="white", color_bg="yellow", mode=1)
    print("The optional submodules are: ", end="")

    for mod in module_name_list[:-1]:
        printInColor('[{}]'.format(mod), mode=1, color_bg='', end='')
        print(", ", end='')
    printInColor('[{}]'.format(module_name_list[-1]), color_bg='', mode=1)

    print("Use command ", end="")  # end设置不换行输出
    printInColor("pip install yangke", color_fg='red',
                 color_bg='yellow', end='', mode=1)
    printInColor("[Database]", color_bg='yellow', end="", mode=1)
    print("/", end='')
    printInColor("pip install *.whl", color_fg='red',
                 color_bg='yellow', end='', mode=1)
    printInColor("[Database]", color_bg='yellow', end="", mode=1)
    print(" to install the selected submodule.")

    print("Use command ", end="")
    printInColor(
        "pip install *.whl[All]", color_fg='red', color_bg='yellow', end='', mode=1)
    printInColor("[Database]", color_bg='yellow', end="", mode=1)
    print(" to install all the submodule.")


def yangke_test():
    info()


def version():
    info()


def test():
    info()


def stock():
    """
    http://y.mairui.club/hslt/list/8B92E04E-302A-4BB0-9468-54FBB51F7401
    8B92E04E-302A-4BB0-9468-54FBB51F7401
    """
    license = '8B92E04E-302A-4BB0-9468-54FBB51F7401'
    url = f'http://y.mairui.club/hslt/list/{license}'


def login(args):
    """
    查询登录用户信息是否正确，返回注册结果
    """
    import sqlalchemy
    from yangke.dataset.YKSqlalchemy import SqlOperator
    import datetime
    username = args['username']
    encrypt_pass = args['password']
    sql: SqlOperator = args['sql']
    try:
        res = sql.select_in_table(table_name='user', condition_dict={'username': username, 'password': encrypt_pass},
                                  result_type='json')

        if res:  # 成功登录后，余额减1
            today = datetime.date.today()
            last_login_time: datetime.datetime = res.get("last_login")
            if last_login_time is None:
                last_login_time = datetime.datetime.now()

            last_login_date = last_login_time.date()
            if last_login_date != today:  # 如果最后登录日期不是今天，则余额减1
                money = res.get("money") - 1
            else:
                money = res.get("money")

            sql.update_item(table_name='user', conditions={'username': username, 'password': encrypt_pass},
                            values={"money": money, "last_login": datetime.datetime.now()})
            return {
                "success": True,
                "login_res_type": 0,
                "login_info": res,
            }
        else:
            return {
                "success": True,
                "login_res_type": 1,
                "login_info": res
            }
    except sqlalchemy.exc.PendingRollbackError:
        traceback.print_exc()
        return {
            "success": False,
            "login_res_type": 2,
            "login_info": traceback.format_exc()
        }
    except:
        traceback.print_exc()
        return {
            "success": False,
            "login_res_type": 3,
            "login_info": traceback.format_exc()
        }


def register(args):
    """
    注册用户，向mysql数据库表中添加用户
    """
    from yangke.dataset.YKSqlalchemy import SqlOperator
    import datetime
    sql: SqlOperator = args['sql']
    username = args['username']
    encrypt_pass = args['password']
    email = args['email']
    money = 0
    vip = 0
    last_login = datetime.datetime.now()

    if sql.exists_in_table('user', condition_dict={'username': username}):
        return {
            "success": True,
            "register_res_type": 1,
            "register_info": "duplicated username"
        }
    elif sql.exists_in_table("user", condition_dict={'email': email}):
        return {
            "success": True,
            "register_res_type": 2,
            "register_info": "duplicated email"
        }
    else:
        sql.insert_item('user', values=[username, email, encrypt_pass, money, vip, last_login],
                        col_names=['username', 'email', 'password', "money", "vip", "last_login"])
        return {"success": True,
                "register_res_type": 0,
                "register_info": "success", }


def charge(args):
    """
    向指定账号充值
    """
    from yangke.dataset.YKSqlalchemy import SqlOperator
    from yangke.common.config import logger
    logger.debug(args)
    name = args.get('name')
    username = name.split('-')[0]
    money = args.get('money')  # 单位元
    money = int(float(money)) * 10  # 将单位转换成角
    pid = args.get('pid')
    no = args.get('out_trade_no')
    site_name = args.get('sitename')
    trade_status = args.get('trade_status')
    sql: SqlOperator = args.get('sql')
    logger.debug(f"{username}, {pid}, {no}, {site_name}, {trade_status}")

    # https://z-pay.cn/submit.php?money=1&name=undefined-SGES-1%E5%85%83out_trade_no=1733882848665&pid=2024120914335319
    try:
        exist_no = sql.select_item('trade', {"no": no})
        logger.debug(f"{exist_no=}, {trade_status=}")
        if trade_status == "TRADE_SUCCESS" and not exist_no:
            now_money = int(sql.select_item(
                'user', {"username": username}, result_col='money'))
            logger.debug(f"{now_money=}")
            conditions = {"username": username}
            values = {"money": now_money + money}
            logger.debug(f"{conditions=}, {values=}")
            sql.update_item('user', conditions, values)
            sql.insert_item('trade', values=[no, username, money, name, pid, trade_status],
                            col_names=['no', 'username', 'amount', 'name', 'pid', 'status'])
            logger.debug(f"充值成功")
            return "success"
        else:
            return "success"  # 告诉支付官方通知收到，不要再次发送了
    except:
        logger.debug(traceback.format_exc())
        return "error"  # 告诉官方再次尝试发送充值信息


def testrest(args):
    return {
        "success": True,
        "info": '服务正在运行中...',
        "args": str(args)
    }


def GetPoolInfo(args):
    from yangke.dataset.YKSqlalchemy import SqlOperator
    sql: SqlOperator = args['sql']
    return {
        "success": True,
        "info": sql.get_pool_info()
    }


def systemTick(args):
    from yangke.dataset.YKSqlalchemy import SqlOperator
    import datetime
    sql: SqlOperator = args.get('sql')
    username = args.get('username')
    try:
        last_login, money = sql.select_item(
            'user', {"username": username}, result_col=['last_login', 'money'])
        if last_login.date() == datetime.date.today():
            now_money = int(money)  # 当天已经登录，则不进行操作
        else:
            # 当前未登录，则扣费后将最后登录时间更改为今天
            now_money = int(money) - 1
            sql.update_item('user', {"username": username},
                            values={'money': now_money, 'last_login': datetime.datetime.now()})
        if now_money < -10:
            return {
                "username": username,
                "isLogin": True,
                "money": now_money,
                "status": "arrear"
            }
        else:
            return {
                "username": username,
                "isLogin": True,
                "money": now_money,
                "status": "success"
            }
    except Exception as e:
        traceback.print_exc()
        return {
            "username": username,
            "isLogin": False,
            "status": "Server Error"
        }


def autoUpdateStockData():
    """
    自动更新股票数据
    """
    ...


def start_restful_mysql(mysql_user, mysql_password, mysql_host='localhost', mysql_port=3306, mysql_db='sges',
                        rest_port=5000, ssl=True, cors=True, single_thread=False):
    from yangke.dataset.YKSqlalchemy import SqlOperator
    from sqlalchemy import create_engine
    from sqlalchemy.pool import QueuePool
    from yangke.web.flaskserver import start_server_app
    sql = SqlOperator(
        create_engine(f'mysql+pymysql://{mysql_user}:{mysql_password}@{mysql_host}:{mysql_port}/{mysql_db}',
                      poolclass=QueuePool,
                      pool_size=5,
                      max_overflow=10,
                      pool_timeout=30,  # 连接池中没有可用连接时的等待时间，超过该时间还没有可用连接，则报错
                      pool_recycle=3600,  # 连接池中的连接1小时后会回收
                      pool_pre_ping=True,  # 每次连接请求时，先测试返回的连接是否还存活，不存活则从连接池中取下一个连接
                      ))

    def deal(args):
        try:
            # 因为下方use_action=True，所以这里的action必然有值，避免eval函数出错
            action = args.get('action')
            args['sql'] = sql
            result = eval(f"{action}(args)")
            return result
        except:
            return {"success": False,
                    "info": "执行deal时错误"}

    start_server_app(deal=deal, use_action=True,
                     allow_action=['login', 'register', 'testrest', 'GetPoolInfo', 'charge',
                                   'systemTick'],
                     host='0.0.0.0',
                     port=rest_port,
                     example_url=[
                         f'http://localhost:{rest_port}/?Action=login&username=杨可&password=test'],
                     single_thread=single_thread, ssl=ssl, cors=cors)


def start_update_stocks_data(kind, ip=None, port=None, user=None, passwd=None, db=None):
    """
    每天定时更新服务器端的股票数据，并提供Restful API服务，默认服务端口5002，
    支持['downloadHolidays', 'downloadStocksDailyAll', 'downloadStocksDailySingle', 'downloadStocksTotalInfo', 'downloadLastDay']等方法。
    
    
    :param kind: 数据库类型，目前支持mysql
    :param ip: 数据库IP地址
    :param port: 数据库端口号
    :param user:
    :param passwd:
    :param db:
    """
    from yangke.stock.dataset.服务器更新股票数据 import UpdateDataBase

    udb = UpdateDataBase(kind=kind, ip=ip, port=port,
                         user=user, passwd=passwd, db=db)
    udb.update()
    udb.start(True)

# 该文件无法调试，提示空套件错误
# if __name__ == '__main__':
#     start_restful_mysql('root', 'YangKe.08', '101.37.118.81')



import os

from flask import Flask
from sqlalchemy import create_engine
from yangke.web.flaskserver import start_server_app, run
from yangke.dataset.YKSqlalchemy import SqlOperator


def deal(args):
    action = args.get('Action')  # 因为下方use_action=True，所以这里的action必然有值，避免eval函数出错
    result = eval("{}(args)".format(action))
    return result


# app = start_server_app(deal=None, run_immediate=False)
sql = SqlOperator(create_engine('mysql+pymysql://sges:sges@sges.yangke.site:3306/sges'))


def login(args):
    username = args['username']
    password = args['password']
    res = sql.exists_in_table('user', condition_dict={'user_name': username, 'password': password})
    return {
        "success": True,
        "login_info": res
    }


def register(args):
    username = args['username']
    password = args['password']
    email = args['email']
    sql.insert_item('user', values=[username, email, password], col_names=['user_name', 'email', 'password'])
    return {"success": True}


app = start_server_app(deal=deal, allow_action=['login', 'register'], host='0.0.0.0', port=5000,
                       example_url=['http://localhost:5000/?Action=login&username=杨可&password=test'])



import time

import win32con
import win32gui
import win32api
from ctypes import windll, byref
from ctypes.wintypes import HWND, POINT
import string
from yangke.common.QtImporter import Qt

dmCharKey = {  # dm插件的键名
    '\x1b': "esc",
    "\x08": "back",
    "\x09": "tab",
    "\x0d": "return",
    "\x10": "shift",
    "\x11": "control",
    "\x12": "menu",
    "\x14": "cap",
}

CodeToKey = {  # dm插件键码
    8: "backspace",
    9: "tab",
    13: "enter",
    16: "shift",
    17: "ctrl",
    18: "alt",
    20: "cap",  # 幽灵键鼠为"capslock"
    27: "esc",
    32: "space",
    33: "1",  # !
    34: "'",  # "
    35: "3",  # #
    36: "4",  # $
    37: "5",  # %
    38: "7",  # &
    39: "'",  # '
    40: "9",  # (
    41: "0",  # )
    42: "8",  # *
    43: "=",  # +
    44: ",",
    45: "-",
    46: ".",
    47: "/",
    48: "0",
    49: "1",
    50: "2",
    51: "3",
    52: "4",
    53: "5",
    54: "6",
    55: "7",
    56: "8",
    57: "9",
    58: ";",  # :
    59: ";",
    60: ",",  # <
    61: "=",
    62: ".",  # >
    63: "/",  # ?
    64: "2",  # 实际是@字符，但按键不考虑shift，一律认为@字符为2
    65: "A",
    66: "B",
    67: "C",
    68: "D",
    69: "E",
    70: "F",
    71: "G",
    72: "H",
    73: "I",
    74: "J",
    75: "K",
    76: "L",
    77: "M",
    78: "N",
    79: "O",
    80: "P",
    81: "Q",
    82: "R",
    83: "S",
    84: "T",
    85: "U",
    86: "V",
    87: "W",
    88: "X",
    89: "Y",
    90: "Z",
    91: "[",
    92: "\\",
    93: "]",  # option
    94: "6",  # ^
    95: "-",  # _
    96: "`",
    97: "a",
    98: "b",
    99: "c",
    100: "d",
    101: "e",
    102: "f",
    103: "g",
    104: "h",
    105: "i",
    106: "j",
    107: "k",
    108: "l",
    109: "m",
    110: "n",
    111: "o",
    112: "p",
    113: "q",
    114: "r",
    115: "s",
    116: "t",
    117: "u",
    118: "v",
    119: "w",
    120: "x",
    121: "y",
    122: "z",
    127: "delete",

}


VkCode = {
    '\x03': 0x03,  # PyQt6界面上按下Ctrl组合键时会产生该键码，意义未知
    "back": 0x08,
    '\x08': 0x08,  # PyQt传递的控制字符过来直接是控制字符的值，不是名称
    "tab": 0x09,
    '\x09': 0x09,
    "return": 0x0D,
    '\x0D': 0x0D,
    "shift": 0x10,
    "control": 0x11,
    "menu": 0x12,
    "pause": 0x13,
    "cap": 0x14,
    '\x14': 0x14,
    "escape": 0x1B,
    "esc": 0x1B,
    '\x1B': 0x1B,
    "space": 0x20,
    '\x20': 0x20,
    "end": 0x23,
    '\x23': 0x23,
    "home": 0x24,
    "\x24": 0x24,
    "left": 0x25,
    "\x25": 0x25,
    "up": 0x26,
    "\x26": 0x26,
    "right": 0x27,
    "\x27": 0x27,
    "down": 0x28,
    "\x28": 0x28,
    "print": 0x2A,
    '\x2A': 0x2A,
    "snapshot": 0x2C,
    "insert": 0x2D,
    '\x2D': 0x2D,
    "delete": 0x2E,
    '\x2E': 0x2E,
    "lwin": 0x5B,
    '\x5B': 0x5B,
    "rwin": 0x5C,
    '\x5C': 0x5C,
    "numpad0": 0x60,
    "numpad1": 0x61,
    "numpad2": 0x62,
    "numpad3": 0x63,
    "numpad4": 0x64,
    "numpad5": 0x65,
    "numpad6": 0x66,
    "numpad7": 0x67,
    "numpad8": 0x68,
    "numpad9": 0x69,
    "multiply": 0x6A,
    "add": 0x6B,
    "separator": 0x6C,
    "subtract": 0x6D,
    "decimal": 0x6E,
    "divide": 0x6F,
    "f1": 0x70,
    "f2": 0x71,
    "f3": 0x72,
    "f4": 0x73,
    "f5": 0x74,
    "f6": 0x75,
    "f7": 0x76,
    "f8": 0x77,
    "f9": 0x78,
    "f10": 0x79,
    "f11": 0x7A,
    "f12": 0x7B,
    "numlock": 0x90,
    "scroll": 0x91,
    "lshift": 0xA0,
    "rshift": 0xA1,
    "lcontrol": 0xA2,
    "rcontrol": 0xA3,
    "lmenu": 0xA4,
    "rmenu": 0XA5
}
PostMessageW = windll.user32.PostMessageW
MapVirtualKeyW = windll.user32.MapVirtualKeyW
VkKeyScanA = windll.user32.VkKeyScanA
ClientToScreen = windll.user32.ClientToScreen
WM_MOUSEMOVE = 0x0200
WM_LBUTTONDOWN = 0x0201
WM_LBUTTONUP = 0x202
WM_MOUSEWHEEL = 0x020A
WHEEL_DELTA = 120
WM_KEYDOWN = 0x100
WM_KEYUP = 0x101


def click_key(handle: HWND, key: str):
    """
    按下指定按键，测试成功

    Args:
        handle (HWND): 窗口句柄
        key (str): 按键名
    """
    vk_code = get_virtual_keycode(key)
    scan_code = MapVirtualKeyW(vk_code, 0)
    # https://docs.microsoft.com/en-us/windows/win32/inputdev/wm-keydown
    wparam = vk_code
    lparam = (scan_code << 16) | 1
    PostMessageW(handle, WM_KEYDOWN, wparam, lparam)
    time.sleep(0.5)
    lparam = (scan_code << 16) | 0XC0000001
    PostMessageW(handle, WM_KEYUP, wparam, lparam)


def get_virtual_keycode(key: str):
    """根据按键名获取虚拟按键码

    Args:
        key (str): 按键名

    Returns:
        int: 虚拟按键码
    """
    if len(key) == 1 and key in string.printable:
        # https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-vkkeyscana
        return VkKeyScanA(ord(key)) & 0xff  # ==ord(key)
    else:
        return VkCode[key]


import sys

if not windll.shell32.IsUserAnAdmin():
    # 不是管理员就提权
    windll.shell32.ShellExecuteW(
        None, "runas", sys.executable, __file__, None, 1)



# 使用baostock库获取股票数据
from datetime import datetime, date

import baostock as bs
import pandas as pd

from yangke.common.config import logger


def get_holiday(day_datetime: datetime.date = date.today(), start_date: date | None = None,
                end_date: date | None = None) -> pd.DataFrame:
    """
    获取股票交易日历
    :param day_datetime:
    :param start_date: 开始日期最早不能早于1990-12-19日，因为A股首次开盘的日期是1990-12-19日
    :param end_date: 结束日期和开始日期之间最多不能超过10000天，建议不要超过20年
    """
    year = day_datetime.year
    bs.login()
    if start_date is not None and end_date is not None:
        start_str = start_date.strftime('%Y-%m-%d')
        end_str = end_date.strftime('%Y-%m-%d')
        holiday_data: pd.DataFrame = bs.query_trade_dates(start_date=start_str,
                                                          end_date=end_str).get_data()
    else:
        holiday_data: pd.DataFrame = bs.query_trade_dates(start_date=f'{year}-01-01',
                                                          end_date=f'{year}-12-31').get_data()
    bs.logout()
    if len(holiday_data) == 0:
        logger.error('获取假日数据出错，检查baostock版本')
    holiday_data.rename(columns={"calendar_date": "date", "is_trading_day": "open"}, inplace=True)
    # 日期必须转换为datetime才能与数据库中的DATE列匹配
    holiday_data["date"] = pd.to_datetime(holiday_data["date"])
    holiday_data["open"] = holiday_data["open"].apply(lambda x: True if int(x) == 1 else False)
    return holiday_data



from flask import Flask, request, make_response
import hashlib
import xml.etree.ElementTree as ET
from yangke.common.config import logger, add_file_logger
from yangke.web.flaskserver import start_server_app
from yangke.web.socket_server import socket_communication_server, YkSocketServer

socket: YkSocketServer | None = None


# app = Flask(__name__)


# @app.route('/', methods=['GET', 'POST'])
# def wechat_auth():
#     logger.debug(f"receive request")
#     if request.method == 'GET':
#         logger.debug("接收到GET请求")
#         token = '5d9d54c798119aab5dfc126ea45432a2'  # 替换为自己在公众号设置中的Token
#         data = request.args
#         signature = data.get('signature', '')
#         timestamp = data.get('timestamp', '')
#         nonce = data.get('nonce', '')
#         echostr = data.get('echostr', '')
#         list = [token, timestamp, nonce]
#         list.sort()
#         s = ''.join(list).encode('utf-8')
#         if (hashlib.sha1(s).hexdigest() == signature):
#             return make_response(echostr)
#         else:
#             return 'Invalid Signature'
#     else:
#         logger.debug("接收到POST请求")
#         xml_data = request.stream.read()
#         xml_tree = ET.fromstring(xml_data)
#         msg_type = xml_tree.find('MsgType').text
#         if msg_type == 'text':
#             content = xml_tree.find('Content').text
#             response_content = chat_with_gpt(content)
#             response = generate_text_response(xml_tree, response_content)
#             return make_response(response)
#         else:
#             response_content = '暂不支持该类型消息的回复。'
#             response = generate_text_response(xml_tree, response_content)
#             return make_response(response)

def on_receive_msg(msg):
    """
    接收到服务器的消息时，执行该方法，如果是转发请求，需要在该方法中将msg发送给需要转发的对象计算机

    Parameters
    ----------
    msg

    Returns
    -------

    """
    # todo 将msg转发给发送请求的计算机
    logger.debug(f"接收到客户端的消息：{msg}")


def deal(text: dict):
    """
    将请求参数发送给远程服务器，由远程服务器处理，本机只做数据转发
    Parameters
    ----------
    text

    Returns
    -------

    """
    global socket
    if socket is not None:
        res = socket.get_answer_of_send(text)  # 当访问http://{ipv4}:{port_of_ipv4}/时，text为空字典{}
        return res
    socket.send(text)
    return {"error": "未知结果"}


def http_transfer_server(http_listen_port=80, transfer_listen_port=9990):
    """
    http请求转发方法，默认将请求到本机80端口的数据转发给远程服务器9990端口，无需指定远程服务器ip，但需要本机具有公网ipv4地址，远程服务器和
    http请求发送者都会通过本机ipv4地址建立连接

    Parameters
    ----------
    http_listen_port: 接受http请求的端口号
    transfer_listen_port: websocket转发请求信息的端口号，本端口与另一台真正提供服务的服务器通信

    Returns
    -------

    """
    global socket
    # 初始化socket，在本机建立websocket通讯，使用9990端口与真正的服务器进行通讯，后续可以使用socket.send方法转发数据
    socket = socket_communication_server(call_back=on_receive_msg, port=transfer_listen_port)

    logger.debug(
        f"http请求转发方法，将请求到本机{http_listen_port}端口的数据转发给远程服务器{transfer_listen_port}端口，无需指定远程服务器ip，但需要本机具有公网ipv4地址，远程服务器和http请求发送者都会通过本机ipv4地址建立连接\n")
    logger.debug(f"正在监听0.0.0.0:{http_listen_port}")

    # 在本机80端口启动服务，使互联网上的计算机可以请求该服务，在接收到请求后，在deal方法中将请求转发给真正的服务器
    start_server_app(deal=deal, host="0.0.0.0", port=http_listen_port, use_action=False)


if __name__ == "__main__":
    add_file_logger('log.txt')
    http_transfer_server(http_listen_port=8080)
    # 测试url: http://localhost:80/?Action=ChatGPT&Question=核电站给水流量孔板的精度是多少？



#!/usr/bin/python3
# 文件名：server.py

# 导入 socket、sys 模块
import socket
import sys
import time

from yangke.base import start_threads, execute_function_by_interval
from yangke.common.config import logger
from queue import Queue  # Queue是线程安全的，因此可以替代我们对操作进行加锁
import uuid

from yangke.web.flaskserver import start_server_app


def deal(msg):
    print(f"Server: 回调函数收到消息{msg}")


class YkSocketServer:
    def __init__(self, port=9990, host='localhost', callback=None):
        """
        创建并初始化一个服务器监听端口。使用示例：
sock = YkSocketServer(callback=deal) # deal为回调函数，该类会自动将端口接收到的消息传递给deal的第一个参数，参考本模块的deal方法
sock.start_server(damon=False)
# 等待连接后，可以调用send方法发送消息
sock.send('服务器消息')

        Parameters
        ----------
        port
        host
        callback: 服务器接收到客户端消息时的回调函数
        """
        self.host = "0.0.0.0" if host == "localhost" else host  # socket.gethostname()在linux上可能报错：socket.gaierror: [Errno -2] Name or service not known
        self.port = port
        # 创建 socket 对象
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # 绑定端口号
        self.server_socket.bind((self.host, self.port))
        print(f"监听{self.host}:{self.port}...")

        # 设置最大连接数，超过后排队
        self.server_socket.listen(5)  # 只能连接一个客户端

        self.connected_sockets = None
        self.callback = callback  # 当端口上接收到客户端发送的消息时，会回调该方法
        self.answers = Queue()  # Queue是线程安全的，在多线程中，会自动对资源进行锁的获取与释放

    def forward(self, msg):
        """
        当服务器接收到客户端消息时，会自动执行该方法
        Parameters
        ----------
        msg

        Returns
        -------

        """
        msg = eval(msg)  # 将返回的msg字符串转换为dict对象
        self.answers.put(msg)
        if self.answers.qsize() > 100:  # 保证Queue对象中的消息不超过100个
            for i in range(50):
                self.answers.get()  # 只保留最后50条消息
        if self.callback is None:
            self.callback = deal
        self.callback(msg)

    def start_server(self, damon=True):
        """
        开启socket服务监听，等待客户端连接
        Parameters
        ----------
        damon： 是否守护线程，如果不守护，将在新进程里开启监听，该方法不会阻塞，否则该方法阻塞监听端口连接

        Returns
        -------

        """
        start_threads(self.on_msg_received)  # 开启新线程，持续检测端口是否接收到新的数据，并执行deal函数对收到的数据进行处理
        self.send("服务器发送的消息！！YK")
        logger.debug("等待[计算服务器]连接，连接后才可以处理http请求")
        if damon:
            while True:
                # 建立客户端连接
                client_socket, addr = self.server_socket.accept()  # 会阻塞，如果连接进来，则clientsocket会返回一个可通信的对象
                self.connected_sockets = client_socket
                logger.debug("[计算服务器]已连接，连接地址: %s" % str(addr))
                msg = 'connect done'
                client_socket.send(msg.encode('utf-8'))
        else:
            def _start_server():
                while True:
                    # 建立客户端连接
                    client_socket, addr = self.server_socket.accept()  # 会阻塞，如果连接进来，则clientsocket会返回一个可通信的对象
                    self.connected_sockets = client_socket
                    logger.debug("[计算服务器]已连接，连接地址: %s" % str(addr))
                    client_socket.send('connect done！'.encode('utf-8'))

            start_threads(_start_server)

    def on_msg_received(self):
        print(f"开始监听端口，并接收消息，如收到消息，会回调{self.callback}")
        while True:
            # 接收小于 1024 字节的数据
            if self.connected_sockets is not None:
                msg = self.connected_sockets.recv(1024)
                self.forward(msg.decode('utf8'))

    def send(self, msg):
        msg = str(msg).encode('utf8')
        if self.connected_sockets is not None:
            self.connected_sockets.send(msg)

    def get_answer_by_id(self, aid):
        answers = list(self.answers.queue)
        for a in answers:  # 因为on_message_received线程在修改self.answers，因此可能需要对资源加锁
            if a.get("mid") == aid:
                return a
        return None

    def get_answer_of_send(self, msg: dict):
        """
        向客户端发送消息，并获取客户端的回复，调用该方法时，不能设置callback参数，必须callback==None

        Parameters
        ----------
        msg        http请求里的参数的字典

        Returns
        -------

        """
        message_id = uuid.uuid1().hex
        msg.update({"mid": message_id})
        msg = str(msg)  # 将int/float/dict等类型统一转换为字符串
        if self.connected_sockets is None:
            logger.warning(f"公网ipv4服务器已经接收到客户端的请求，但尚未收到真正的内网服务器的连接请求，因此无法与内网服务器建立连接！")
            logger.warning(f"请先启动【内网服务器】上的socket客户端，以完成http请求的最终处理！")
            return {"Error": "请先启动【内网服务器】上的socket客户端，以完成http请求的最终处理！"}
        self.send(msg)
        # 循环扫描接收到的消息，找出与本次发送相对应的，并返回
        while self.get_answer_by_id(message_id) is None:  # 此处虽然在查询self.answers，但是并不占用锁
            time.sleep(1)
        answer = self.get_answer_by_id(message_id)
        return answer


def socket_communication_server(port=9990, call_back=None):
    """
    开启websocket通信，监听本机9990端口，如果本机接受到请求，可以通过该接口将请求转发至另一台电脑。
    转发请求时。使用socket.send(msg)即可，本方法不会阻塞主进程

    Returns
    -------

    """
    socket1 = YkSocketServer(port=port, callback=call_back)
    socket1.start_server(damon=False)  # 启动后持续监听本机指定端口，不阻塞
    return socket1


if __name__ == "__main__":
    sock = socket_communication_server(call_back=deal, port=9990)  #
    while True:
        sock.send("server 1")
        time.sleep(5)



from transformers import AutoTokenizer, AutoModel, AutoModelForSequenceClassification

checkpoint="distilbert/distilbert-base-uncased-finetuned-sst-2-english"
tokenizer=AutoTokenizer.from_pretrained(checkpoint)

raw_input = [
    "I've been waiting for a this course my whole life.",
    "I hate this so much!"
]

# padding=True, 是否需要将长度补齐成一样的
# truncation=True, 截断，限制最大长度，单独设置无效，很多模型的输入长度上限是512
# return_tensors="pt"，指底层用pytorch
inputs = tokenizer(raw_input,padding=True, truncation=True, return_tensors="pt")

decoder_input_id = inputs["input_ids"][1]
decode_input = tokenizer.decode(decoder_input_id)

print(f"{inputs=}")
print(f"{decode_input=}")

# model = AutoModel.from_pretrained(checkpoint)
#
# print(f"{model=}")
#
# outputs = model(**inputs)
# print(f"{outputs.last_hidden_state.shape=}")

model = AutoModelForSequenceClassification.from_pretrained(checkpoint)


# from yangke import start_update_stocks_data
# from yangke.base import yield_all_file
# from yangke.objDetect.ocr import ocr
#
# start_update_stocks_data('mysql', '101.37.118.81', '3306', 'root', 'YangKe.08', 'stocks')

import threading
import time
from functools import wraps


def timeout(seconds=5, callback=None):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            # 定义一个线程局部变量来存储函数结果
            result = None
            exception = None

            def target():
                nonlocal result, exception
                try:
                    result = func(*args, **kwargs)
                except Exception as e:
                    exception = e

            # 创建并启动线程
            thread = threading.Thread(target=target)
            thread.start()

            # 等待线程完成，最多等待 seconds 秒
            thread.join(seconds)

            # 如果线程仍然存活，说明函数执行超时
            if thread.is_alive():
                if callback is not None:
                    callback()  # 调用回调函数
                raise TimeoutError(f"Function {func.__name__} timed out after {seconds} seconds")

            # 如果函数抛出了异常，重新抛出
            if exception is not None:
                raise exception

            return result

        return wrapper

    return decorator


# 使用示例
def timeout_callback():
    print("Timeout occurred! Executing callback...")


@timeout(seconds=5)
def long_running_function():
    time.sleep(3)
    return "Function completed"


try:
    result = long_running_function()
    print(result)
except TimeoutError as e:
    print(e)



import os
import time
import traceback

from PyQt6.QtGui import QPixmap
from PyQt6.QtWidgets import QDialog

from yangke.common.QtImporter import (QKeyEvent, QAction, Qt, QHBoxLayout, QMessageBox, QKeyEvent, QVBoxLayout,
                                      QPushButton)

from yangke.common.qt import (YkWindow, run_app, logger, QApplication, QWidget, QLabel, QComboBox,
                              QLineEdit, YkItem, YkInputPanel, YkScrollArea, YkImageWidget, YkDialog, layout_to_widget)
from yangke.common.win.dminput import DMRemote
from yangke.common.win.keyboard import VkCode, CodeToKey
from yangke.common.win.win_x64 import get_all_window
from yangke.game.gf import Step, Steps, Frame, NotExist, Exist, Region, AnchorRegion, RectRegion, Position, Offset, \
    RequestWeb
from yangke.base import start_threads, stop_threads, show_pic, pic2qlabel, pic2qpixmap
import pyautogui
from PIL import Image, ImageGrab
import yangke.game.ghost.ghostbox as gb
import win32con


class StepWidget(YkItem):
    def __init__(self, idx, op, target, judge, condition, name, root_window):
        self.name = name
        self.step = Step(op, target, judge, condition)
        self.first_item = YkItem(f"步骤{idx}", f"<button on-click='btn_clicked'>删除步骤{idx}</button>",
                                 f"<button on-click='btn_clicked'>运行步骤{idx}</button>",
                                 size=[50, 20, 20])
        self.first_item.value.clicked.connect(root_window.btn_clicked)
        self.first_item.unit.clicked.connect(root_window.btn_clicked)
        self.op_data = {
            "click key": ["键位描述"],
            "press key": ["键位描述"],
            "release key": ["键位描述"],
            "left click": ["画面中的文字", "画面中的图片", "相对偏移", "自定义对象"],
            "right click": ["画面中的文字", "画面中的图片", "相对偏移", "自定义对象"],
            "double click": ["画面中的文字", "画面中的图片", "相对偏移", "自定义对象"],
        }

        self.item11 = QComboBox()
        self.item11.addItems(self.op_data.keys())
        # noinspection all
        self.item11.currentTextChanged.connect(self.change_op)
        self.item12 = QComboBox()
        self.item12.addItems(self.op_data["click key"])
        self.item12.currentTextChanged.connect(self.change_op_dest)
        self.item13_1: QLabel | None = None
        self.item13_2 = QLineEdit("")
        self.item13 = YkItem(None, self.item13_2, None, margins=[0, 0, 0, 0], size=[0, 50, 0], struct=0)

        self.item1 = YkItem(self.item11, self.item12, unit=self.item13,
                            size=[20, 20, 50])
        self.item21 = QComboBox()
        self.item21.addItems(["重复", "等待", "无"])
        self.item22 = QComboBox()
        self.item22.addItems(["直到"])
        self.item23 = QComboBox()
        self.item23.addItems(["无", "存在文字", "存在图片", "不存在文字", "不存在图片", "自定义条件"])
        self.item23.currentTextChanged.connect(self.change_condition)
        self.condition_item = YkItem(label=self.item21,
                                     value=self.item22,
                                     unit=self.item23,
                                     size=[20, 20, 50],
                                     margins=(10, 0, 10, 0))
        self.item31 = None
        self.item31_1: QLabel = QLabel("预览图")
        self.item31_2: QLineEdit = QLineEdit()
        self.prefer_obj_line: QLineEdit = None
        self.range_line = None
        self.bak_item = None
        super().__init__(label=self.first_item, value=self.item1, unit=self.condition_item, bak=self.bak_item,
                         direction="v", bgcolor="green")
        self.set_step(self.step)

    def set_step(self, step):
        try:
            self.item11.setCurrentText(step.op)
            if step.op in ["click key", "press key", "release key"]:
                self.item12.setCurrentText("键位描述")
                if isinstance(step.target, dict):
                    self.item13_2.setText(step.target.get("value"))
                else:
                    self.item13_2.setText(step.target)
            else:
                if isinstance(step.target, Offset):
                    self.item12.setCurrentText("相对偏移")
                    self.item13_1.setText(step.target.anchor_obj)
                    self.item13_2.setText(f"{step.target.dx},{step.target.dy}")
                elif step.target.get("__cls__") == "Offset":
                    ...
                elif step.target.get("__cls__") == "Picture":
                    self.item12.setCurrentText("画面中的图片")
                    self.item13_2.setText(step.target["path"])
                    self.item13_1.setPixmap(QPixmap(step.target["path"]))
                elif step.target.get("__cls__") == "Text":
                    self.item12.setCurrentText("画面中的文字")
                    self.item13_2.setText(step.target["text"])
            if step.judge == "wait":
                self.item22.setCurrentText("等待")
            elif step.judge == "repeat":
                self.item22.setCurrentText("重复")
            else:
                self.item22.setCurrentText("无")
            if step.condition is None:
                self.item23.setCurrentText("无")
            elif isinstance(step.condition, Exist):
                if step.condition.type_ == "text":
                    self.item23.setCurrentText("存在文字")
                    if isinstance(step.condition.value, list) and len(step.condition.value) == 1:
                        text = step.condition.value[0]
                        self.item31.setText(text)
                elif step.condition.type_ == "pic":
                    self.item23.setCurrentText("存在图片")
                    if isinstance(step.condition.value, list) and len(step.condition.value) == 1:
                        path = step.condition.value[0]
                        self.item31_2.setText(path)
                        self.item31_1.setPixmap(QPixmap(path))
            elif isinstance(step.condition, NotExist):
                if step.condition.type_ == "text":
                    self.item23.setCurrentText("不存在文字")
                    if isinstance(step.condition.value, list) and len(step.condition.value) == 1:
                        text = step.condition.value[0]
                        self.item31.setText(text)
                elif step.condition.type_ == "pic":
                    self.item23.setCurrentText("不存在图片")
                    if isinstance(step.condition.value, list) and len(step.condition.value) == 1:
                        path = step.condition.value[0]
                        self.item31_2.setText(path)
                        self.item31_1.setPixmap(QPixmap(path))
            else:
                self.item23.setCurrentText("自定义条件")
                self.prefer_obj_line.setText(step.condition.value)

            if isinstance(step.condition, Exist) or isinstance(step.condition, NotExist):
                if step.condition.anchor_name is not None:
                    self.prefer_obj_line.setText(step.condition.anchor_name)
                if step.condition.region is not None:
                    if isinstance(step.condition.region, list):
                        _ = [str(__) for __ in step.condition.region]
                        self.range_line.setText(",".join(_))


        except:
            logger.warning(f"StepWidget加载出错")

    def form_bak_item(self):
        condition = self.condition_item.get_value_and_unit()[1]
        if condition == "无":
            self.bak_item = None
            return self.bak_item
        elif condition == "存在文字" or condition == "不存在文字":
            self.item31 = QLineEdit()
            self.item31.setPlaceholderText("输入条件判断的文字")
        elif condition == "自定义条件":
            self.item31 = None
            self.prefer_obj_line = QLineEdit()
            self.prefer_obj_line.setPlaceholderText("输入自定义条件")
            self.range_line = None
            self.bak_item = YkItem(label=self.item31, value=self.prefer_obj_line, unit=self.range_line,
                                   size=[0, 90, 0], margins=[1, 10, 0, 10])
            return self.bak_item
        else:
            self.item31_1 = QLabel("预览图")
            self.item31_2 = QLineEdit("")
            self.item31_2.setPlaceholderText("图片路径")
            self.item31 = YkItem(self.item31_1, self.item31_2, f"<button on-click='capture_mini_pic'>截图</button>",
                                 size=[16, 22, 12], margins=[0, 0, 0, 0], struct=2, parent=self)
            self.item31.unit.clicked.connect(self.capture_mini_pic)
        self.prefer_obj_line = QLineEdit()
        self.prefer_obj_line.setPlaceholderText("输入参考点定义")
        self.range_line = QLineEdit()
        self.range_line.setPlaceholderText("输入查找区域定义")
        self.bak_item = YkItem(label=self.item31, value=self.prefer_obj_line, unit=self.range_line,
                               size=[50, 20, 20])

        return self.bak_item

    def get_step(self):
        op = self.item11.currentText()
        if self.item12.currentText() == "相对偏移":
            target = {
                "__cls__": "Offset",
                "value": self.item13.get_value(),
                "anchor": self.item13.get_label_text()
            }
        elif self.item12.currentText() == "画面中的图片":
            target = {
                "__cls__": "Pic",
                "value": self.item13.get_value(),
                "anchor": self.item13.get_label_text()
            }
        elif self.item12.currentText() == "画面中的文字":
            target = {
                "__cls__": "Text",
                "value": self.item13.get_value(),
            }
        elif self.item12.currentText() == "键位描述":
            target = {
                "__cls__": "Key",
                "value": self.item13.get_value(),
            }
        else:  # "自定义对象"
            target = {
                "__cls__": "UserDefObj",
                "value": self.item13.get_value(),
            }
        judge = self.item21.currentText()
        cond = self.item23.currentText()
        if self.item31 is not None:
            if isinstance(self.item31, QLineEdit):
                dest_obj = self.item31.text()
            elif isinstance(self.item31, YkItem):
                dest_obj = self.item31.get_value()
            else:
                dest_obj = None
        else:
            dest_obj = None
        judge = {"重复": "repeat", "等待": "wait", "无": None}.get(judge)
        anchor_name = None
        dest_range = None
        if self.prefer_obj_line is not None and self.prefer_obj_line.text() is not None:
            anchor_name = self.prefer_obj_line.text()
        if self.range_line is None:
            _ = None
        else:
            _ = self.range_line.text()
        if _ is not None and _.strip() != "":
            dest_range = _.strip().split(",")
            try:
                dest_range = [int(_) for _ in dest_range]
            except:
                logger.error(f"目标区域中存在非数值类型字符{dest_range=}")

        if cond == "无":
            condition = None
        elif cond == "存在文字":
            condition = Exist(dest_obj, type_="text", region=dest_range, anchor_name=anchor_name)
        elif cond == "不存在文字":
            condition = NotExist(dest_obj, type_="text", region=dest_range, anchor_name=anchor_name)
        elif cond == "存在图片":
            condition = Exist(dest_obj, type_="pic", region=dest_range, anchor_name=anchor_name)
        elif cond == "不存在图片":
            condition = NotExist(dest_obj, type_="pic", region=dest_range, anchor_name=anchor_name)
        else:  # cond == "自定义条件"
            condition = self.item31.get_value()
        return Step(op=op, target=target, judge=judge, condition=condition)

    def change_op(self):
        op_name = self.item11.currentText()
        self.item12.clear()
        self.item12.addItems(self.op_data[op_name])

    def change_op_dest(self):
        op_type = self.item12.currentText()
        if op_type == "画面中的文字":
            self.item13.clear_all()
            self.item13.add_item(0, "")
            self.item13_2 = QLineEdit("")
            self.item13.add_item(1, self.item13_2)
            self.item13_2.setPlaceholderText("输入点击的文字")
            self.item13.set_size({"width": [0, 50, 0]})
        elif op_type == "键位描述":
            self.item13.clear_all()
            self.item13.add_item(0, "")
            self.item13_2 = QLineEdit("")
            self.item13.add_item(1, self.item13_2)
            self.item13_2.setPlaceholderText("输入键盘按键或组合键描述")
            self.item13.set_size({"width": [0, 50, 0]})
        elif op_type == "画面中的图片":
            self.item13.clear_all()
            self.item13_1 = QLabel("预览图")
            self.item13.add_item(0, self.item13_1)
            self.item13_2 = QLineEdit("")
            self.item13_2.setPlaceholderText("图片路径")
            self.item13.add_item(1, self.item13_2)
            self.item13.add_item(2, f"<button on-click='capture_mini_pic'>截图</button>")
            self.item13.set_size({"width": [16, 22, 12]})
            self.item13.unit.clicked.connect(self.capture_mini_pic)
        elif op_type == "相对偏移":
            self.item13.clear_all()
            self.item13_1 = QLineEdit("")
            self.item13_1.setPlaceholderText("输入偏移对象")
            self.item13_1.setToolTip("例如：step1.target/step1.condition")
            self.item13.add_item(0, self.item13_1)
            self.item13_2 = QLineEdit("")
            self.item13.add_item(1, self.item13_2)
            self.item13_2.setPlaceholderText("输入偏移参数")
            self.item13_2.setToolTip("例如：300, 100表示偏移dx=300, dy=200")
            self.item13.set_size({"width": [25, 25, 0]})
        elif op_type == "自定义对象":
            self.item13.clear_all()
            self.item13.add_item(0, "")
            self.item13_2 = QLineEdit("")
            self.item13.add_item(1, self.item13_2)
            self.item13_2.setPlaceholderText("")
            self.item13.set_size({"width": [0, 50, 0]})

    def change_condition(self):
        self.remove_item(3)
        self.add_item(3, self.form_bak_item())

    def capture_mini_pic(self):
        sender = self.sender()
        if isinstance(sender, QPushButton):
            text = sender.text()
            if text == "截图":
                pyautogui.hotkey("win", "shift", "s")
                sender.setText("粘贴")
            elif text == "粘贴":
                parent = sender.parent().parent()
                if isinstance(parent, YkItem) and parent.get_value() == "画面中的图片":
                    # 说明是鼠标点击的图片
                    step_name = sender.parent().parent().parent().label.label.text()  # 步骤1
                    file_name = f"{self.name}_{step_name}_click_obj.png"
                else:
                    # 说明是条件判断的图片
                    step_name = sender.parent().parent().parent().label.label.text()  # 步骤1
                    file_name = f"{self.name}_{step_name}_condition_obj.png"
                im = ImageGrab.grabclipboard()
                if isinstance(im, Image.Image):
                    im.save(file_name)
                    sender.parent().value.setText(file_name)
                    label: QLabel = sender.parent().label
                    label.setPixmap(QPixmap(file_name))
                    label.setScaledContents(True)
                elif im:
                    for filename in im:
                        print("filename:%s" % filename)
                        im = Image.open(filename)
                else:
                    print("clipboard is empty")

                sender.setText("截图")


class ChooseWindow(QDialog):

    def __init__(self, parent=None, windows: dict = {}):
        """
        选择要操作的窗口，从当前所有窗口中
        :param windows: 格式为{hwnd: title}的窗口信息字典
        """
        super().__init__(parent)
        self.setFixedWidth(600)
        self.setFixedHeight(600)
        self.setWindowTitle("选择值")
        self.layout = QVBoxLayout(self)

        self.combo_box = QComboBox(self)

        self.combo_box.addItems(list(windows.values()))  # 注意这里我们添加的是字符串，但你可以根据需要处理它们

        self.layout.addWidget(self.combo_box)

        self.ok_button = QPushButton("确定", self)
        self.ok_button.clicked.connect(self.accept)
        self.layout.addWidget(self.ok_button)

    def get_selected_value(self):
        # 返回选中的值（作为字符串）
        return self.combo_box.currentText()


class MainWindow(YkWindow):

    def __init__(self):
        super().__init__()
        self.temp_ = None
        self.add_input_panel("ui/ui_panel.yaml", domain="综合")
        _ = self.proj.get("综合")
        if isinstance(_, dict):
            for k, v in _.items():
                self._input_panel.set_value(k, v)
        self.thread = None
        self.running = False
        # self.set_status_bar_label("天谕")
        self.frame: Frame | None = None
        self.add_input_panel("ui/ui_panel.yaml", domain="自定义步骤")
        panel: YkInputPanel = self.panels.get("自定义步骤")
        step_name = panel.get_values_and_units(need_unit=False)[0].strip()
        if step_name is None or step_name == "":
            panel.set_value("步骤序列名称", "自定义任务1")
            step_name = "自定义任务1"
        else:
            panel.set_value("步骤序列名称", step_name)
        if self.proj.get("temp_steps") is None or self.proj.get("temp_steps") == []:
            item = StepWidget(1, "press", "R", None, None, step_name, self)
            if panel is not None:
                panel.insert_item(1, item)
        else:
            for idx, step in enumerate(self.proj.get("temp_steps")):
                item = StepWidget(idx=idx + 1, root_window=self, name=step_name, **step)
                panel.insert_item(idx + 1, item)
            # self.add_content_tab(YkItem(), "存在条件设置")
        self.add_content_tab(QLabel("."), "欢迎")
        self.capture_ykimagewidget = YkImageWidget([], True, self.do_action)
        scroll = YkScrollArea()
        scroll.setWidget(self.capture_ykimagewidget)
        self.add_content_tab(widget=scroll, tab_name="CT截图")

        # self.statusBar1.addPermanentWidget()

        # self.add_content_tab(widget=self.capture_ykimagewidget, tab_name="CT截图") # 直接使用该widget会导致每次截图，画面高度增加，貌似是QT的bug
        self._keyboard = False  # 模拟键盘功能是否开启
        self._mouse = False  # 模拟鼠标功能是否开启
        if self.proj.get("dm_sim_info") is None:
            self.proj.update({"dm_sim_info": {"key": "null",
                                              "add_key": "",
                                              "display": "gid2",
                                              "mouse": "dx.mouse.position.lock.api",
                                              "keypad": "windows",
                                              "public": "",
                                              "mode": "101:超级绑定模式.可隐藏目标进程中的dll，推荐使用",
                                              "port": 8765},
                              "sim_mode": "DM-remote"},
                             )
        if self.proj.get("temp_steps") is None:
            self.proj.update({
                "temp_steps": []
            })
        if self.proj.get("steps_repos") is None:
            self.proj.update({
                "steps_repos": {}
            })

        if self.proj.get("RequestWebPort") is not None:
            self.request_web = RequestWeb(self.proj.get("RequestWebPort"))
        else:
            self.request_web = RequestWeb(port=8765)

        self.mode_repos = {
            "天谕": {
                "display": "dx.graphic.3d.10plus",
                "mouse": "dx.mouse.state.api|dx.mouse.api|dx.mouse.cursor",
                "keypad": "dx.keypad.state.api|dx.keypad.api",
                "public": "dx.public.graphic.protect|dx.public.anti.api|dx.public.km.protect|dx.public.prevent.block",
                "mode": "101:超级绑定模式.可隐藏目标进程中的dll，推荐使用",
            },
            "天谕1": {
                "display": "dx.graphic.3d.10plus",
                "mouse": "dx.mouse.position.lock.api|dx.mouse.position.lock.message|dx.mouse.clip.lock.api|dx.mouse.input.lock.api|dx.mouse.state.api|dx.mouse.api|dx.mouse.cursor|dx.mouse.raw.input",
                "keypad": "dx.keypad.input.lock.api|dx.keypad.state.api|dx.keypad.api",
                "public": "",
                "mode": "0:推荐模式此模式比较通用，而且后台效果是最好的",
            },
            "笑傲江湖": {

            },
            "剑叁": {
                "display": "dx.graphic.3d.10plus",
                "mouse": "dx.mouse.api|dx.mouse.raw.input",
                "keypad": "dx.keypad.input.lock.api|dx.keypad.state.api|dx.keypad.api",
                "public": "",
                "mode": "0:推荐模式此模式比较通用，而且后台效果是最好的",
            },
            "逆水寒": {
                "display": "dx.graphic.3d.10plus",
                "mouse": "dx.mouse.state.api|dx.mouse.api|dx.mouse.cursor",
                "keypad": "dx.keypad.state.api|dx.keypad.api",
                "public": "dx.public.graphic.protect|dx.public.anti.api|dx.public.km.protect|dx.public.prevent.block",
                "mode": "101:超级绑定模式.可隐藏目标进程中的dll，推荐使用",
            },
            "魔兽世界": {
                "display": "dx.graphic.3d.10plus",
                "mouse": "dx.mouse.clip.lock.api|dx.mouse.state.api|dx.mouse.api|dx.mouse.cursor",
                "keypad": "dx.keypad.input.lock.api|dx.keypad.state.api|dx.keypad.api",
                "public": "dx.public.hide.dll|dx.public.anti.api|dx.public.km.protect",
                "mode": "101:超级绑定模式.可隐藏目标进程中的dll，推荐使用",
            }
        }

    def show_dialog(self):
        dialog = ChooseWindow(self, get_all_window())
        if dialog.exec() == QDialog.DialogCode.Accepted:
            selected_value = dialog.get_selected_value()
            self.panels.get("综合").set_value("游戏ID", selected_value)
            self.init_frame(force_update=True)
            self.capture_window()

    def yk_signal_received(self, msg: dict):
        if msg.get("action") == "capture_window":
            self.capture_window()

    def set_sim_mode(self):
        _ = os.path.abspath(os.path.join(os.path.dirname(__file__), "ui", "ui_panel.yaml"))
        input_panel = YkInputPanel(from_file=_, domain="设置键鼠模拟方式", parent=self)

        dialog = YkDialog(self, widget=input_panel, modality=False)
        dialog.set_size(600, 500)
        input_panel.apply_btn_connect()
        self.temp_ = {"input_panel": input_panel, "dialog": dialog}

        dm_sim_info = self.proj.get("dm_sim_info")
        input_panel.set_value("注册码", value=dm_sim_info.get("key") or "null")
        input_panel.set_value("附加码", value=dm_sim_info.get("add_key") or "")
        input_panel.set_value("显示", value=dm_sim_info.get("display") or "gdi2")
        input_panel.set_value("鼠标", value=dm_sim_info.get("mouse") or "dx.mouse.position.lock.api")
        input_panel.set_value("键盘", value=dm_sim_info.get("keypad") or "windows")
        input_panel.set_value("公共参数", value=dm_sim_info.get("public") or "")
        input_panel.set_value("模式", value=dm_sim_info.get("mode"))
        try:
            input_panel.set_value("端口", value=int(dm_sim_info.get("port")) or 8765)
        except:
            QMessageBox.warning(None, "错误提示", f"端口号必须是整数，当前为({dm_sim_info.get('port')})")
            input_panel.set_value("端口", value=8765)

        input_panel.set_value("模拟方式", self.proj.get("sim_mode"))


    def _change_sim_mode(self):
        input_panel: YkInputPanel = self.temp_.get("input_panel")
        type_ = input_panel.get_item("模拟方式").get_value()
        input_panel.remove_item(index=list(range(1, input_panel.get_items_count() - 2)))
        tails = -2  # 面板尾部的固定YkItem数量
        if type_ == "DM-remote" or type_.startswith("DM-local"):
            input_panel.get_item("模拟方式").unit.setHidden(False)
            input_panel.get_item("模拟方式").unit.setText("加载DM")
            dm_sim_info = self.proj.get("dm_sim_info")
            input_panel.insert_item(tails, YkItem("注册码", "", "", size=[30, 130, 0]))  # 在倒数第三个位置插入YkItem
            input_panel.set_value("注册码", value=dm_sim_info.get("key") or "null")
            input_panel.insert_item(tails, YkItem("附加码", "", "", size=[30, 130, 0]))
            input_panel.set_value("附加码", value=dm_sim_info.get("add_key") or "")
            input_panel.insert_item(tails, YkItem("显示", "", "", size=[30, 130, 0]))
            input_panel.set_value("显示", value=dm_sim_info.get("display") or "gdi2")
            input_panel.insert_item(tails, YkItem("鼠标", "", "", size=[30, 130, 0]), )
            input_panel.set_value("鼠标", value=dm_sim_info.get("mouse") or "dx.mouse.position.lock.api")
            input_panel.insert_item(tails, YkItem("键盘", "", "", size=[30, 130, 0]), )
            input_panel.set_value("键盘", value=dm_sim_info.get("keypad") or "windows")
            input_panel.insert_item(tails, YkItem("公共参数", "", "", size=[30, 130, 0]), )
            input_panel.set_value("公共参数", value=dm_sim_info.get("public") or "")
            input_panel.insert_item(tails, YkItem("模式", "", "", size=[30, 130, 0]), )
            input_panel.set_value("模式",
                                  value=dm_sim_info.get("mode") or "101:超级绑定模式.可隐藏目标进程中的dll，推荐使用")
            input_panel.insert_item(tails, YkItem("端口", "", "", size=[30, 130, 0]), )
            input_panel.set_value("端口", value=dm_sim_info.get("port") or 8765)
            item: YkItem = self.panels["综合"].get_item("游戏ID")
            item.value.setEnabled(True)
            item.unit.setHidden(False)
        elif type_ == "Normal":
            input_panel.get_item("模拟方式").unit.setHidden(True)
            item: YkItem = self.panels["综合"].get_item("游戏ID")

            item.value.setEnabled(True)
            item.unit.setHidden(False)

        elif type_ == "幽灵键鼠":
            input_panel.get_item("模拟方式").unit.setHidden(False)
            input_panel.get_item("模拟方式").unit.setText("测试幽灵键鼠")

            item: YkItem = self.panels["综合"].get_item("游戏ID")
            item.value.setEnabled(True)
            item.unit.setHidden(False)
        elif type_ == "易键鼠":
            input_panel.get_item("模拟方式").unit.setHidden(True)

            item: YkItem = self.panels["综合"].get_item("游戏ID")
            item.value.setEnabled(True)
            item.unit.setHidden(False)
        elif type_ == "易键鼠-双头远程":
            input_panel.get_item("模拟方式").unit.setHidden(True)
            item: YkItem = self.panels["综合"].get_item("游戏ID")
            item.set_value("USB视频流-易键鼠-远程操作")
            item.value.setEnabled(False)
            item.unit.setHidden(True)
        elif type_ == "幽灵键鼠-双头远程":
            input_panel.get_item("模拟方式").unit.setHidden(True)
            item: YkItem = self.panels["综合"].get_item("游戏ID")
            item.set_value("USB视频流-幽灵键鼠-远程操作")
            item.value.setEnabled(False)
            item.unit.setHidden(True)

    def add_step(self):
        panel: YkInputPanel = self.panels.get("自定义步骤")
        idx = panel.get_items_count()
        name = panel.get_item("步骤序列名称").value.text()
        panel.insert_item(idx, StepWidget(idx, "press", "R", None, None, name, self))

    def init(self):
        self.init_frame()

        self.frame.init_chat(anchor="ui/ZhaoMu.png", anchor_region=AnchorRegion(0, "anchor.y1", "anchor.x1", -10),
                             channels=["世界", "团队", "队伍", "附近", "阵营", "地区"])
        suc = self.frame.init_task(anchor="任务", find_region=RectRegion(left=-350, top=300, right=-1, bottom=-500),
                                   anchor_region=AnchorRegion("anchor.x1", "anchor.y1", -10, "anchor.y1+400"))
        if not suc:
            self.frame.init_task(anchor="驻地", find_region=RectRegion(left=-350, top=300, right=-1, bottom=-500),
                                 anchor_region=AnchorRegion("anchor.x1-100", "anchor.y1", -10, "anchor.y1+400"))

        # self.frame.init_time(region=(-170, 0, -2, 30))
        self.frame.init_time(region=(-25, 5, -2, 25))
        # self.frame.turn_direction_to("青蛙", region=Region(width_child=1000, height_child=800))

    def run(self):
        # self.init()
        # settings = self.get_value_of_panel(need_dict=True, need_unit=False)
        # key = settings.get("打怪技能按键")
        # role = settings.get("游戏ID").strip()
        # mode = settings.get("打怪模式")
        # freq = float(settings.get("按键频率"))
        # # self.frame = Frame(role)
        # # link_pos = self.frame.task.get_text_link_pos_global("打开公会")
        # # self.frame.show_region(link_pos)
        # #
        # # self.frame.left_click(*link_pos.get_center(), offset=(16, 0))
        # # self.frame.dm.unbind_window()
        # if mode == "无脑打怪":
        #     steps_重复按键 = Steps(
        #         steps=[
        #             Step("press", key, None, None),
        #         ]
        #     )
        # else:
        #     steps_重复按键 = Steps(
        #         steps=[
        #             Step("press", key, "until",
        #                  NotExist("竹林偷伐者", last_time=20, region=Region(align="center", width_child=600)),
        #                  wait_method="repeat"),
        #             Step("double-press", "space", "until",
        #                  Exist("竹林偷伐者", last_time=60, interval=10, region=Region(align="center", width_child=600)),
        #                  wait_method="repeat"),
        #         ]
        #     )
        # self.thread = start_threads(self.frame.run_steps_forever, args_list=[steps_重复按键, freq])
        self._input_panel.get_button("运行").setDisabled(True)
        self._input_panel.get_button("停止").setDisabled(False)

    def stop(self):
        stop_threads(self.thread)
        self.running = False
        logger.debug(f"停止任务")
        self._input_panel.get_button("停止").setDisabled(True)
        self._input_panel.get_button("运行").setDisabled(False)
        if self.proj.get("sim_mode") == "DM-remote":
            self.frame.dm.unbind_window()
            if self._mouse:
                self._mouse = False
            if self._keyboard:
                self._keyboard = False

    def save(self):
        settings = self.get_value_of_panel(need_dict=True, need_unit=False, domain="综合")
        self.proj.update({"综合": settings})
        super().save()

    def init_frame(self, force_update=False):
        """
        绑定需要操作的窗口界面

        :params force_update： 已有绑定窗口时是否强制更新绑定的窗口
        """
        if self.frame is None or self.frame.window is None or force_update or not self.frame.bind_success:
            settings = self.get_value_of_panel(need_dict=True, need_unit=False, domain="综合")
            role = settings.get("游戏ID").strip()
            if self.frame is not None:
                if hasattr(self.frame, "dm") and self.frame.bind_success:
                    self.frame.dm.unbind_window()
            self.frame = Frame(role,
                               sim_mode=self.proj.get("sim_mode"),
                               sim_info=self.proj.get("dm_sim_info"))
            if self.frame.window is None:
                self.statusBar1.showMessage("未找到游戏窗口")
                return None
        return self.frame

    def capture_window(self):

        self.init_frame()
        if self.frame.window is None:
            return
        if not self.frame.bind_success:
            QMessageBox.information(self, "提示", "使用DM-remote插件，但窗口绑定失败")
        snapshot = self.frame.capture_window_xywh()
        # screen = QApplication.primaryScreen()
        # snapshot = screen.grabWindow(self.frame.window).toImage()
        # show_pic(snapshot)
        if len(snapshot) == 0:  # snapshot == []
            return
        if self.capture_ykimagewidget is None:
            self.capture_ykimagewidget = YkImageWidget(snapshot, True, self.do_action)
            scroll = YkScrollArea()
            scroll.setWidget(self.capture_ykimagewidget)
            self.add_content_tab(widget=scroll, tab_name="CT截图")
            # self.add_content_tab(widget=self.capture_ykimagewidget, tab_name="CT截图")
        else:
            self.capture_ykimagewidget.replace_image(snapshot)

        if "CT截图" in self._content_tab.labels and self._content_tab.get_current_tab_name() != "CT截图":
            self._content_tab.activate_tab("CT截图")
        pos = self.frame.get_cursor_pos()
        if pos is not None:
            self.set_status_bar_label(str(pos))
        self.statusBar1.showMessage("就绪")

    def real_time_capture(self):
        logger.info("开始实时截图")
        while self._mouse or self._keyboard:
            self.yk_signal.emit({"action": "capture_window"})
            time.sleep(0.1)
        logger.info("实时截图结束")
        self.statusBar1.showMessage("就绪")

    def btn_clicked(self, anything=None, anything2=None, **kwargs):
        """
        在截图区域上点击时的响应时间
        """
        sender = self.sender()
        text = sender.text()
        if text == "截图":
            self.capture_window()
            self.statusBar1.showMessage("就绪")

        if text == "加载DM":
            sim_info = self.proj.get("dm_sim_info")
            dm_remote = DMRemote(key=sim_info.get("key"), add_key=sim_info.get("add_key"), port=sim_info.get("port"))
            if dm_remote.success:
                QMessageBox.information(None, "提示", "DM-remote加载成功")
            else:
                QMessageBox.information(None, "提示", f"DM-remote加载失败，{dm_remote.error_info}")
        elif text == "测试幽灵键鼠":
            res = gb.opendevice(0)
            if gb.isconnected():
                QMessageBox.information(self, "提示", "幽灵键鼠设备正常")
            else:
                QMessageBox.information(self, "提示", "幽灵键鼠设备打开失败，请检查硬件是否存在")
        elif text == "保存键鼠模拟方式":
            input_panel: YkInputPanel = self.temp_.get("input_panel")
            values = input_panel.get_values_and_units(need_unit=False, need_dict=True, need_label=True)
            if self.proj.get("sim_mode") != values.get("模拟方式"):  # 更改模拟方式后，重新绑定窗口
                self.proj.update({"sim_mode": values.get("模拟方式")})
                self.init_frame(force_update=True)
            if values.get("模拟方式") == "DM-remote" or values.get("模拟方式") == "DM-local":
                try:
                    port = values.get("端口")
                except:
                    QMessageBox(None, "错误提示", f"端口号必须是整数，当前为({values.get('端口')})")
                    port = self.proj.get("dm_sim_info").get("port") or 8765
                self.proj.get("dm_sim_info").update({"key": values.get("注册码"),
                                                     "add_key": values.get("附加码"),
                                                     "display": values.get("显示"),
                                                     "mouse": values.get("鼠标"),
                                                     "keypad": values.get("键盘"),
                                                     "public": values.get("公共参数"),
                                                     "mode": values.get("模式"),
                                                     "port": port,
                                                     })
            else:
                pass
            # self.temp_: YkDialog
            if isinstance(self.temp_.get("dialog"), YkDialog):
                self.temp_.get("dialog").close()
            self.statusBar1.showMessage("就绪")

        elif text == "绑定窗口":
            if self.init_frame() is None:
                QMessageBox.information(self, "提示", "绑定失败，详细信息请检查日志输出")
            else:
                QMessageBox.information(self, "提示", "窗口绑定成功")
            self.statusBar1.showMessage("就绪")
        elif text == "移动鼠标":
            # 测试移动鼠标功能
            self.init_frame()
            _x, _y = self.frame.window_pos
            w, h = self.frame.window_size
            x1 = int(_x + w / 2)
            y1 = int(_y + h / 2)
            # x1, y1 = 963, 102
            x2, y2 = x1 + 100, y1 + 11
            interval = int((x2 - x1) / 10)
            self.frame.left_press(x1, y1)
            QApplication.processEvents()
            for i in range(10):
                self.frame.move_to(x1 + i * interval, y1 + i)
                logger.debug(f"dest pos: {x1 + i * interval}, {y1}  cursor pos: {self.frame.get_cursor_pos()}")
                QApplication.processEvents()
            self.frame.left_release(x1 + 21, y1)

        elif text.startswith("运行步骤"):
            self.statusBar1.showMessage("开始执行步骤")
            panel: YkInputPanel = self.panels.get("自定义步骤")
            idx = int(text.replace("运行步骤", ""))
            step_widget: StepWidget = panel.get_item(f"步骤{idx}")
            step: Step = step_widget.get_step()
            self.frame.run_steps(Steps([step]))
            self.statusBar1.showMessage("步骤执行完毕")
        elif text.startswith("删除步骤"):
            panel: YkInputPanel = self.panels.get("自定义步骤")
            panel.remove_item(-1)
        elif text.startswith("测试键盘"):
            # 测试键盘事件
            input_panel: YkInputPanel = self.temp_.get("input_panel")
            cmd = input_panel.get_item("测试键盘").get_value()  # click_key('A')
            cmd = f"self.frame.{cmd}"
            try:
                exec(cmd)
            except Exception as e:
                traceback.print_exc()
            self.capture_window()
        elif text.startswith("测试鼠标"):
            # 测试鼠标模拟是否正常
            input_panel: YkInputPanel = self.temp_.get("input_panel")
            cmd = input_panel.get_item("测试鼠标").get_value()  # left_click(10,10)
            cmd = f"self.frame.{cmd}"
            try:
                exec(cmd)
            except Exception as e:
                traceback.print_exc()
            self.capture_window()

    def load_steps(self, combox: QComboBox | str, dialog: YkDialog):
        """
        加载指定名称的步骤序列
        """
        if isinstance(combox, QComboBox):
            steps_name = combox.currentText()
        else:
            steps_name = combox
        steps = self.proj.get("steps_repos").get(steps_name)

        dialog.close()

    def close_load_steps_repos(self, dialog: YkDialog):
        self.statusBar1.showMessage("就绪")
        dialog.close()

    def steps_clicked(self):
        sender = self.sender()
        text = sender.text()
        if text == "加载":
            steps_repos = self.proj.get("steps_repos")
            if steps_repos is not None and len(steps_repos) > 0:
                steps_name = list(steps_repos.keys())
                button1 = QPushButton("确定")
                button2 = QPushButton("取消")

                choose_steps = QComboBox()
                choose_steps.addItems(steps_name)

                vbox = QVBoxLayout()
                vbox.addWidget(choose_steps)
                vbox.addWidget(button1)
                vbox.addWidget(button2)
                dialog = YkDialog(self, "加载步骤序列", layout_to_widget(vbox))
                dialog.set_size(300, 300)

                button1.clicked.connect(lambda: self.load_steps(choose_steps, dialog))
                button2.clicked.connect(lambda: self.close_load_steps_repos(dialog))
        elif text == "运行":
            if self.running:
                QMessageBox.warning(self, "警告", "已经有任务在运行了！")
                return
            self.init_frame()
            panel: YkInputPanel = self.panels.get("自定义步骤")
            steps = []
            for i in range(1, panel.get_items_count()):
                _step = panel.get_item(f"步骤{i}")
                self.statusBar1.showMessage(f"执行步骤{i}")
                step = _step.get_step()
                steps.append(step)

            # self.frame.run_steps(Steps(steps))
            self.thread = start_threads(self.frame.run_steps, (Steps(steps),))  # 新线程中运行步骤序列，否则界面卡死
            self.running = True
        elif text == "保存":
            panel: YkInputPanel = self.panels.get("自定义步骤")
            steps_name = panel.get_item("步骤序列名称").get_value()
            steps_json = []
            for i in range(1, panel.get_items_count()):
                _step = panel.get_item(f"步骤{i}")
                step = _step.get_step()
                steps_json.append(step.to_json())
            self.proj["temp_steps"] = steps_json
            self.proj["steps_repos"].update({steps_name: steps_json})
            self.statusBar1.showMessage(f"步骤序列 [{steps_name}] 保存成功")

    def do_action(self, x, y, op):
        """
        执行操作，然后等待1s，截图显示
        """
        if self._mouse:  # 如果开启了鼠标模拟测试，则转发鼠标操作至绑定的窗口
            if self.frame is not None and self.frame.window is not None:
                x_inner = int(self.capture_ykimagewidget.x_scale * x)
                y_inner = int(self.capture_ykimagewidget.y_scale * y)
                if op == "left_click":
                    self.frame.left_click(x_inner, y_inner)
                elif op == "right_click":
                    self.frame.right_click(x_inner, y_inner)
                elif op == "left_press":
                    logger.debug(f"left_press {x_inner}, {y_inner}")
                    self.frame.left_press(x_inner, y_inner)
                elif op == "right_press":
                    logger.debug(f"right_press {x_inner}, {y_inner}")
                    self.frame.right_press(x_inner, y_inner)
                elif op == "left_release":
                    logger.debug(f"left_release {x_inner}, {y_inner}")
                    self.frame.left_release(x_inner, y_inner)
                elif op == "right_release":
                    logger.debug(f"right_up {x_inner}, {y_inner}")
                    self.frame.right_release(x_inner, y_inner)
                elif op == "move":
                    logger.debug(f"move {x_inner}, {y_inner}")
                    self.frame.move_to(x_inner, y_inner)
        else:
            if self.frame is not None and self.frame.window is not None:
                x_inner = int(self.capture_ykimagewidget.x_scale * x)
                y_inner = int(self.capture_ykimagewidget.y_scale * y)
                self.set_status_bar_label(f"{x_inner}, {y_inner}")

    def event(self, event):
        if isinstance(event, QKeyEvent):
            key = event.nativeVirtualKey()
            if self._keyboard:
                if key in [win32con.VK_TAB, win32con.VK_F5]:  # and event.type() == QKeyEvent.Type.KeyRelease):
                    self.keyPressEvent(event)  # Qt默认Tab键会被窗口处理，不会传递给KeyPressEvent事件
                    return True  # 该事件已被处理
            else:
                return super().event(event)

        return super().event(event)

    def keyPressEvent(self, a0: QKeyEvent) -> None:
        """
        键盘按键-按下某键
        测试方法只支持短按，不支持按键不放，但按键不放可以在内部实现
        20240702:支持长按
        """
        key_code = a0.nativeVirtualKey()  # 获取按键的标准键码，a0.key()是Qt的特有键码，不通用
        if self._keyboard:  # 如果模拟键盘开启，则拦截所有键盘按键至绑定的窗口
            self.init_frame()
            if key_code == win32con.VK_ESCAPE:
                if self.capture_ykimagewidget.region_show:
                    return  # 释放按键时响应
            key = CodeToKey[key_code]
            self.frame.press_key(key=key)
        else:
            if self.capture_ykimagewidget.region_show:
                if key_code == win32con.VK_ESCAPE:
                    return  # 释放时响应
            super().keyPressEvent(a0)

    def keyReleaseEvent(self, a0: QKeyEvent):
        key_code = a0.nativeVirtualKey()
        if self._keyboard:
            if key_code == win32con.VK_ESCAPE:
                if self.capture_ykimagewidget.region_show:
                    self.capture_ykimagewidget.region_show = False
                    self.capture_window()
                    return  # 按下按键时已经响应
            key = CodeToKey[key_code]
            self.frame.release_key(key=key)
        else:
            if self.capture_ykimagewidget.region_show:
                if key_code == win32con.VK_ESCAPE:
                    self.capture_ykimagewidget.region_show = False
                    self.capture_window()
                    return  # 释放时响应
            super().keyReleaseEvent(a0)

    def show_new_region(self):
        if self.capture_ykimagewidget is None:
            return
        geo = self.capture_ykimagewidget.geometry()
        self.capture_ykimagewidget.region_show = True

    def tool_click(self, k1):
        # 工具栏点击事件
        sender: QAction = self.sender()
        text: str = self.sender().text()
        if text == "模拟键盘":
            if self._keyboard:
                sender.setChecked(False)
                self._keyboard = False
            else:
                sender.setChecked(True)
                self._keyboard = True
                self.panels.get("综合").get_button("停止").setDisabled(False)
                if self._mouse is False:
                    start_threads(self.real_time_capture)
        elif text == "模拟鼠标":
            if self._mouse:
                sender.setChecked(False)
                self._mouse = False
            else:
                sender.setChecked(True)
                self._mouse = True
                self.panels.get("综合").get_button("停止").setDisabled(False)
                if self._keyboard is False:
                    start_threads(self.real_time_capture)

        elif text == "新建区域":
            self.show_new_region()
            self.capture_window()

    def refresh(self):
        self.frame.dm.unbind_window()  # 解绑窗口，重新绑定
        self.init_frame()
        if "CT截图" in self._content_tab.labels:
            self._content_tab.remove_tab("CT截图")
        self.capture_ykimagewidget = YkImageWidget([], True, self.do_action)
        scroll = YkScrollArea()
        scroll.setWidget(self.capture_ykimagewidget)
        self.add_content_tab(widget=scroll, tab_name="CT截图")


run_app(MainWindow)



from yangke.common.qt import YkWindow, run_app
from yangke.common.config import logger
from yangke.common.QtImporter import QPushButton


class MainWindow(YkWindow):
    def __init__(self):
        super().__init__()
        self.enable_input_panel(panel_ui_file=None)
        self.setWindowTitle("中转服务器连接工具", with_project_name=False)

    def button_clicked(self):
        sender = self.sender()  # 发送事件的组件，可能是button、YkDialog等任何拥有signal的类
        if isinstance(sender, QPushButton):
            self.statusBar().showMessage(sender.text() + ' was pressed')
        if sender.text() == "测试连接":
            # 。。
            ...


run_app(MainWindow)
